@@ -3,18 +3,19 @@
 // email : everettjf@live.com
 // created at 2022-01-01
 //
+use crate::demangle;
 use anyhow::{anyhow, Result};
 use gimli::{DW_TAG_subprogram, DebugInfoOffset, Dwarf, EndianSlice, RunTimeEndian};
-use object::{Object, ObjectSection};
+use object::{Object, ObjectSection, ObjectSegment};
 use std::path::Path;
 use std::{borrow, fs};
-use crate::demangle;
 
 pub fn print_addresses(
     object_path: &str,
     load_address: u64,
     addresses: Vec<u64>,
-    debug_mode: bool,
+    verbose: bool,
+    file_offset_type: bool,
 ) -> Result<(), anyhow::Error> {
     let file = fs::File::open(&object_path)?;
     let mmap = unsafe { memmap::Mmap::map(&file)? };
@@ -26,7 +27,7 @@ pub fn print_addresses(
         .ok_or(anyhow!("file name error(to_str)"))?;
 
     if is_object_dwarf(&object) {
-        if debug_mode {
+        if verbose {
             println!("dwarf");
         }
         dwarf_symbolize_addresses(
@@ -34,10 +35,11 @@ pub fn print_addresses(
             object_filename,
             load_address,
             addresses,
-            debug_mode,
+            verbose,
+            file_offset_type,
         )
     } else {
-        if debug_mode {
+        if verbose {
             println!("symbols");
         }
         symbol_symbolize_addresses(
@@ -45,7 +47,8 @@ pub fn print_addresses(
             object_filename,
             load_address,
             addresses,
-            debug_mode,
+            verbose,
+            file_offset_type,
         )
     }
 }
@@ -63,17 +66,37 @@ fn symbol_symbolize_addresses(
     object_filename: &str,
     load_address: u64,
     addresses: Vec<u64>,
-    debug_mode: bool,
+    verbose: bool,
+    file_offset_type: bool,
 ) -> Result<(), anyhow::Error> {
+    // find vmaddr for __TEXT segment
+    let mut segments = object.segments();
+    let mut text_vmaddr = 0;
+    while let Some(segment) = segments.next() {
+        if let Some(name) = segment.name()? {
+            if name == "__TEXT" {
+                text_vmaddr = segment.address();
+                break;
+            }
+        }
+    }
+
     for address in addresses {
-        if debug_mode {
+        if verbose {
             println!("---------------------------------------------");
             println!("BEGIN ADDRESS {} | {:016x}", address, address);
         }
 
-        let symbol_result =
-            symbol_symbolize_address(&object, object_filename, load_address, address, debug_mode);
-        if debug_mode {
+        let symbol_result = symbol_symbolize_address(
+            &object,
+            object_filename,
+            load_address,
+            address,
+            text_vmaddr,
+            verbose,
+            file_offset_type,
+        );
+        if verbose {
             println!("RESULT:")
         }
         match symbol_result {
@@ -81,7 +104,7 @@ fn symbol_symbolize_addresses(
             Err(err) => println!("N/A - {}", err),
         };
 
-        if debug_mode {
+        if verbose {
             println!("END ADDRESS {} | {:016x}", address, address);
         }
     }
@@ -93,9 +116,15 @@ fn symbol_symbolize_address(
     object_filename: &str,
     load_address: u64,
     address: u64,
-    _debug_mode: bool,
+    text_vmaddr: u64,
+    _verbose: bool,
+    file_offset_type: bool,
 ) -> Result<String, anyhow::Error> {
-    let search_address = address - load_address;
+    let search_address: u64 = if file_offset_type {
+        address - load_address
+    } else {
+        address - load_address + text_vmaddr
+    };
 
     let symbols = object.symbol_map();
     let found_symbol = symbols.get(search_address);
@@ -117,7 +146,8 @@ fn dwarf_symbolize_addresses(
     object_filename: &str,
     load_address: u64,
     addresses: Vec<u64>,
-    debug_mode: bool,
+    verbose: bool,
+    file_offset_type: bool,
 ) -> Result<(), anyhow::Error> {
     let endian = if object.is_little_endian() {
         gimli::RunTimeEndian::Little
@@ -135,23 +165,49 @@ fn dwarf_symbolize_addresses(
     })?;
     let dwarf = dwarf_cow.borrow(|section| gimli::EndianSlice::new((&*section).as_ref(), endian));
 
+    // find vmaddr for __TEXT segment
+    let mut segments = object.segments();
+    let mut text_vmaddr = 0;
+    while let Some(segment) = segments.next() {
+        if let Some(name) = segment.name()? {
+            if name == "__TEXT" {
+                text_vmaddr = segment.address();
+                break;
+            }
+        }
+    }
+
     for address in addresses {
-        if debug_mode {
+        if verbose {
             println!("---------------------------------------------");
             println!("BEGIN ADDRESS {} | {:016x}", address, address);
         }
 
-        let symbol_result =
-            dwarf_symbolize_address(&dwarf, object_filename, load_address, address, debug_mode);
-        if debug_mode {
+        let symbol_result = dwarf_symbolize_address(
+            &dwarf,
+            object_filename,
+            load_address,
+            address,
+            text_vmaddr,
+            verbose,
+            file_offset_type,
+        );
+        if verbose {
             println!("RESULT:")
         }
         match symbol_result {
             Ok(symbol) => println!("{}", symbol),
             Err(_) => {
                 // downgrade to symbol table search
-                let symbol_result =
-                    symbol_symbolize_address(&object, object_filename, load_address, address, debug_mode);
+                let symbol_result = symbol_symbolize_address(
+                    &object,
+                    object_filename,
+                    load_address,
+                    address,
+                    text_vmaddr,
+                    verbose,
+                    file_offset_type,
+                );
                 match symbol_result {
                     Ok(symbol) => println!("{}", symbol),
                     Err(err) => println!("N/A - {}", err),
@@ -159,22 +215,27 @@ fn dwarf_symbolize_addresses(
             }
         };
 
-        if debug_mode {
+        if verbose {
             println!("END ADDRESS {} | {:016x}", address, address);
         }
     }
     Ok(())
 }
 
-
 fn dwarf_symbolize_address(
     dwarf: &Dwarf<EndianSlice<RunTimeEndian>>,
     object_filename: &str,
     load_address: u64,
     address: u64,
-    debug_mode: bool,
+    text_vmaddr: u64,
+    verbose: bool,
+    file_offset_type: bool,
 ) -> Result<String, anyhow::Error> {
-    let search_address = address - load_address;
+    let search_address: u64 = if file_offset_type {
+        address - load_address
+    } else {
+        address - load_address + text_vmaddr
+    };
 
     // aranges
     let mut debug_info_offset: Option<DebugInfoOffset> = None;
@@ -194,6 +255,7 @@ fn dwarf_symbolize_address(
 
         if found_address {
             debug_info_offset = Some(header.debug_info_offset());
+            break;
         }
     }
 
@@ -206,7 +268,7 @@ fn dwarf_symbolize_address(
     // catch header
     let debug_info_header = dwarf.debug_info.header_from_offset(debug_info_offset)?;
 
-    if debug_mode {
+    if verbose {
         println!("got debug info header 0x{:016x}", debug_info_offset.0);
     };
 
@@ -220,12 +282,12 @@ fn dwarf_symbolize_address(
                 let mut low_pc: Option<u64> = None;
                 let mut high_pc: Option<u64> = None;
                 if let Ok(Some(gimli::AttributeValue::Addr(lowpc_val))) =
-                entry.attr_value(gimli::DW_AT_low_pc)
+                    entry.attr_value(gimli::DW_AT_low_pc)
                 {
                     low_pc = Some(lowpc_val);
                     let high_pc_value = entry.attr_value(gimli::DW_AT_high_pc);
 
-                    if debug_mode {
+                    if verbose {
                         println!("high pc value : {:?}", high_pc_value);
                     }
 
@@ -238,7 +300,7 @@ fn dwarf_symbolize_address(
                     }
                 }
 
-                if debug_mode {
+                if verbose {
                     println!("low pc = {:?}, high pc = {:?}", low_pc, high_pc)
                 }
 
@@ -255,7 +317,7 @@ fn dwarf_symbolize_address(
             }
         }
     }
-    if debug_mode {
+    if verbose {
         println!("found_symbol_name = {:?}", found_symbol_name);
     };
 
@@ -296,7 +358,7 @@ fn dwarf_symbolize_address(
         }
     }
 
-    if debug_mode {
+    if verbose {
         println!(
             "found_file_name = {:?} found_line = {:?}",
             found_file_name, found_line
@@ -304,7 +366,7 @@ fn dwarf_symbolize_address(
     };
 
     if let (Some(symbol_name), Some(file_name), Some(line)) =
-    (found_symbol_name, found_file_name, found_line)
+        (found_symbol_name, found_file_name, found_line)
     {
         // expect format
         // main (in BinaryName) (main.m:100)
@@ -318,4 +380,3 @@ fn dwarf_symbolize_address(
     }
     Err(anyhow!("failed search symbol"))
 }
-
