@@ -25,9 +25,13 @@ struct Args {
     #[clap(parse(try_from_str = parse_address_string))]
     addresses: Vec<u64>,
 
-    /// Enable debug mode with extra output
-    #[clap(short, default_value_t = 0)]
-    debug: u8,
+    /// Enable verbose mode with extra output
+    #[clap(short)]
+    verbose: bool,
+
+    /// Addresses are file offsets (ignore vmaddr in __TEXT or other executable segment)
+    #[clap(short)]
+    file_offset_type: bool,
 }
 
 fn parse_address_string(address: &str) -> Result<u64, anyhow::Error> {
@@ -44,13 +48,13 @@ fn parse_address_string(address: &str) -> Result<u64, anyhow::Error> {
 fn main() {
     let args = Args::parse();
     let object_path = args.object_path.into_os_string().into_string().unwrap();
-    let debug_mode = args.debug != 0;
 
     let result = atosl::print_addresses(
         &object_path,
         args.load_address,
         args.addresses,
-        debug_mode,
+        args.verbose,
+        args.file_offset_type,
     );
     match result {
         Ok(..) => {}
